@@ -3,13 +3,13 @@ use strict;
 use warnings;
 
 package Bencode;
-$Bencode::VERSION = '1.401';
+$Bencode::VERSION = '1.500';
 # ABSTRACT: BitTorrent serialisation format
 
 use Carp;
 use Exporter::Tidy all => [qw( bencode bdecode )];
 
-our ( $DEBUG, $do_lenient_decode, $max_depth );
+our ( $DEBUG, $do_lenient_decode, $max_depth, $undef_encoding );
 
 sub _msg { sprintf "@_", pos() || 0 }
 
@@ -114,30 +114,28 @@ sub bdecode {
 	return $deserialised_data;
 }
 
+sub _bencode;
 sub _bencode {
-	my ( $data ) = @_;
-	if ( not ref $data ) {
-		return sprintf 'i%se', $data if $data =~ m/\A (?: 0 | -? [1-9] \d* ) \z/x;
-		return length( $data ) . ':' . $data;
-	}
-	elsif ( ref $data eq 'SCALAR' ) {
-		# escape hatch -- use this to avoid num/str heuristics
-		return length( $$data ) . ':' . $$data;
-	}
-	elsif ( ref $data eq 'ARRAY' ) {
-		return 'l' . join( '', map _bencode( $_ ), @$data ) . 'e';
-	}
-	elsif ( ref $data eq 'HASH' ) {
-		return 'd' . join( '', map { _bencode( \$_ ), _bencode( $data->{ $_ } ) } sort keys %$data ) . 'e';
-	}
-	else {
-		croak 'unhandled data type';
-	}
+	map
+	+( ( not defined     ) ? ( $undef_encoding or croak 'unhandled data type' )
+	:  ( not ref         ) ? ( m/\A (?: 0 | -? [1-9] \d* ) \z/x ? 'i' . $_ . 'e' : length . ':' . $_ )
+	:  ( 'SCALAR' eq ref ) ? ( length $$_ ) . ':' . $$_ # escape hatch -- use this to avoid num/str heuristics
+	:  (  'ARRAY' eq ref ) ? 'l' . ( join '', _bencode @$_ ) . 'e'
+	:  (   'HASH' eq ref ) ? 'd' . do { my @k = sort keys %$_; join '', map +( length $k[0] ) . ':' . ( shift @k ) . $_, _bencode @$_{ @k } } . 'e'
+	:  croak 'unhandled data type'
+	), @_
 }
 
 sub bencode {
-	croak 'need exactly one argument' if @_ != 1;
-	goto &_bencode;
+	my $undef_mode = @_ == 2 ? pop : 'str';
+	$undef_mode = 'str' unless defined $undef_mode;
+	local $undef_encoding
+		= 'str' eq $undef_mode ? '0:'
+		: 'num' eq $undef_mode ? 'i0e'
+		: 'die' eq $undef_mode ? undef
+		: croak qq'undef_mode argument must be "str", "num", "die" or undefined, not "$undef_mode"';
+	croak 'need exactly one or two arguments' if @_ != 1;
+	( &_bencode )[0];
 }
 
 bdecode( 'i1e' );
@@ -154,7 +152,7 @@ Bencode - BitTorrent serialisation format
 
 =head1 VERSION
 
-version 1.401
+version 1.500
 
 =head1 SYNOPSIS
 
@@ -166,13 +164,38 @@ version 1.401
 
 =head1 DESCRIPTION
 
-This module implements the BitTorrent I<bencode> serialisation format as described in L<http://www.bittorrent.org/beps/bep_0003.html#bencoding>.
+This module implements the BitTorrent I<bencode> serialisation format,
+as described in L<http://www.bittorrent.org/beps/bep_0003.html#bencoding>.
 
 =head1 INTERFACE
 
-=head2 C<bencode( $datastructure )>
+=head2 C<bencode( $datastructure [, $undef_mode ] )>
 
-Takes a single argument which may be a scalar or a reference to a scalar, array or hash. Arrays and hashes may in turn contain values of these same types. Simple scalars that look like canonically represented integers will be serialised as such. To bypass the heuristic and force serialisation as a string, use a reference to a scalar.
+Takes data to be encoded as a single argument which may be a scalar,
+or may be a reference to either
+a scalar, an array or a hash. Arrays and hashes may in turn contain values of
+these same types. Plain scalars that look like canonically represented integers
+will be serialised as such. To bypass the heuristic and force serialisation as
+a string, use a reference to a scalar.
+
+The second argument is optional (in which case it defaults to C<str>) and
+specifies how to treat C<undef> values. You can pick one of three options:
+
+=over 6
+
+=item C<str>
+
+to encode C<undef>s as empty strings;
+
+=item C<num>
+
+to encode C<undef>s as zeroes;
+
+=item C<die>
+
+to croak upon encountering an C<undef> value.
+
+=back
 
 Croaks on unhandled data types.
 
@@ -180,9 +203,12 @@ Croaks on unhandled data types.
 
 Takes a string and returns the corresponding deserialised data structure.
 
-If you pass a true value for the second option, it will disregard the sort order of dict keys. This violation of the I<bencode> format is somewhat common.
+If you pass a true value for the second option, it will disregard the sort
+order of dict keys. This violation of the I<bencode> format is somewhat common.
 
-If you pass an integer for the third option, it will croak when attempting to parse dictionaries nested deeper than this level, to prevent DoS attacks using maliciously crafted input.
+If you pass an integer for the third option, it will croak when attempting to
+parse dictionaries nested deeper than this level, to prevent DoS attacks using
+maliciously crafted input.
 
 Croaks on malformed data.
 
@@ -210,23 +236,28 @@ Your data includes a string declared to be longer than the available data.
 
 =item C<malformed string length at %s>
 
-Your data contained a string with negative length or a length with leading zeroes.
+Your data contained a string with negative length or a length with leading
+zeroes.
 
 =item C<malformed integer data at %s>
 
-Your data contained something that was supposed to be an integer but didn't make sense.
+Your data contained something that was supposed to be an integer but didn't
+make sense.
 
 =item C<dict key not in sort order at %s>
 
-Your data violates the I<bencode> format constaint that dict keys must appear in lexical sort order.
+Your data violates the I<bencode> format constaint that dict keys must appear
+in lexical sort order.
 
 =item C<duplicate dict key at %s>
 
-Your data violates the I<bencode> format constaint that all dict keys must be unique.
+Your data violates the I<bencode> format constaint that all dict keys must be
+unique.
 
 =item C<dict key is not a string at %s>
 
-Your data violates the I<bencode> format constaint that all dict keys be strings.
+Your data violates the I<bencode> format constaint that all dict keys be
+strings.
 
 =item C<dict key is missing value at %s>
 
@@ -234,21 +265,31 @@ Your data contains a dictionary with an odd number of elements.
 
 =item C<nesting depth exceeded at %s>
 
-Your data contains dicts or lists that are nested deeper than the $max_depth passed to C<bdecode()>.
+Your data contains dicts or lists that are nested deeper than the $max_depth
+passed to C<bdecode()>.
 
 =item C<unhandled data type>
 
-You are trying to serialise a data structure that consists of data types other than
+You are trying to serialise a data structure that consists of data types other
+than
 
 =over
 
-=item * scalars
+=item *
 
-=item * references to arrays
+scalars
 
-=item * references to hashes
+=item *
 
-=item * references to scalars
+references to arrays
+
+=item *
+
+references to hashes
+
+=item *
+
+references to scalars
 
 =back
 
@@ -258,9 +299,9 @@ The format does not support this.
 
 =head1 BUGS AND LIMITATIONS
 
-Strings and numbers are practically indistinguishable in Perl, so C<bencode()> has to resort to a heuristic to decide how to serialise a scalar. This cannot be fixed.
-
-Please report any bugs or feature requests through the web interface at L<http://github.com/ap/Bencode/issues>.
+Strings and numbers are practically indistinguishable in Perl, so C<bencode()>
+has to resort to a heuristic to decide how to serialise a scalar. This cannot
+be fixed.
 
 =head1 AUTHOR
 
@@ -268,7 +309,7 @@ Aristotle Pagaltzis <pagaltzis@gmx.de>
 
 =head1 COPYRIGHT AND LICENSE
 
-This software is copyright (c) 2015 by Aristotle Pagaltzis.
+This software is copyright (c) 2017 by Aristotle Pagaltzis.
 
 This is free software; you can redistribute it and/or modify it under
 the same terms as the Perl 5 programming language system itself.
